# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: component.py,v 1.53 2005/01/11 20:58:09 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import os, sys, math
import glob
import rpmdir
from element import Element
import util, message

# These are the archs that are allowed as a function of key value.
# This is probably not the most flexible way to implement this,
# but it will do for now...
arch_compat = {"noarch":["noarch"],
	       "i386"  :["i386","i586","i686","athlon", "noarch"],
               "x86_64":["i386","i586","i686","athlon","noarch"],
               "athlon":["athlon","noarch"],
               "ppc"   :["ppc","noarch"]}

# Class describing a component of a distribution.  A component consists of
# directories for RPMS and SRPMS, and some files in the base dir.  Most of
# the real work goes on in this class, since the parents are only
# containers of directories, so this file is in essence the "heart" of the
# program.

class Component(Element):

    m = message.Message()
    m.set_indent(3)

    def __init__(self):
        Element.__init__(self)
        self.type = 3
        self.local = False
        self.basedir = None
        self.symlinks = []
        self.hdrfiles = []

    # Method called to set up the list of files maintained by this
    # class. It is called once when both path and id have been
    # set. All files and directories owned by Component has the
    # component id in the name.

    def make_files_list(self):
        self.rpmdir = rpmdir.RPMdir(os.path.join(self.path,"RPMS."+self.id))
        self.rpmdir.issource = False
        self.srpmdir = rpmdir.RPMdir(os.path.join(self.path,"SRPMS."+self.id))
        self.srpmdir.issource = True
        self.rpmdir.parent = self
        self.srpmdir.parent = self
        self.basedir = os.path.join(self.path,"base")
        self.yumdir = os.path.join(self.path,self.id)
        self.hdrdir = os.path.join(self.path,self.id,"headers")
        self.metadir = os.path.join(self.path,self.id,"repodata")
        self.rpmdir.hdrdir = self.hdrdir
        self.srpmdir.hdrdir = self.hdrdir
        self.add(self.rpmdir)
        self.add(self.srpmdir)

        # these directories are owned by me
        self.dirs.append(self.rpmdir.path)
        self.dirs.append(self.srpmdir.path)
        self.dirs.append(self.yumdir)
        self.dirs.append(self.hdrdir)
        self.dirs.append(self.metadir)
        
        # prepend path to these files
        L = ["pkglist."+self.id+".bz2", "srclist."+self.id+".bz2",
             "release."+self.id]
        for f in map(lambda x: os.path.join(self.path,"base",x), L):
            self.files.append(f)
        #;
        
        f = os.path.join(self.basedir,"rpmmacros.%s" % self.id)
        self.files.append(f)

        # shelved md5 sig file
        f = os.path.join(self.basedir,"md5."+self.id)
        self.files.append(f)

        # index file produced by genpkglist
        f = os.path.join(self.basedir,"index."+self.id)
        self.files.append(f)

        # index_<id>.html generated by webgen
        f = os.path.join(self.path,"html","index_"+self.id+".html")
        self.files.append(f)

        # symlinks required by yum
        self.symlinks.append(os.path.join(self.yumdir,"RPMS"))
        self.symlinks.append(os.path.join(self.yumdir,"SRPMS"))

        # header files required by yum
        self.files.append(os.path.join(self.hdrdir,"header.info"))
        self.files.append(os.path.join(self.hdrdir,"header.src.info"))

        # metadata files required by yum
        self.files.append(os.path.join(self.metadir,"repomd.xml"))
        self.files.append(os.path.join(self.metadir,"filelists.xml.gz"))
        self.files.append(os.path.join(self.metadir,"other.xml.gz"))
        self.files.append(os.path.join(self.metadir,"primary.xml.gz"))

        # append the .hdr files in the header dir (if any), this is so
        # they can be checked and destroyed (if user wants)
        for f in glob.glob(os.path.join(self.hdrdir,"*.hdr")):
            self.hdrfiles.append(f)
        #;
    #;


    # Override set_path from base class "Element"

    def set_path(self, path):
        Element.set_path(self,path)
        if self.id and not self.basedir:
            self.make_files_list()
        #;
        return
    #;

    # Override set_id from base class "Element"
    def set_id(self, id):
        Element.set_id(self,id)
        if self.path and not self.basedir:
            self.make_files_list()
        #;
        return
    #;

    def set_arch(self,arch):
        self.arch = arch
        self.rpmdir.archs = arch_compat[arch]
        self.srpmdir.archs = ["src"]
        return
    #;

    # Override set_local from base class, since we need to do something extra,
    # namely to add to our file list. Must be called after set_id and set_path.

    def set_local(self,local):
        self.local = local
        if local:
            self.rpmdir.set_local(True)
            self.srpmdir.set_local(True)
        #;
    #;

    # Check to see if all directories and files are present. We also check the
    # protection mode and correct it if wrong.

    def check(self,level):
        if self.ignore: return
        self.m.info("Checking component %s\n" % self.id)
        result = 0

        if not os.path.exists(self.path):
            self.m.failure("Component directory not found: %s\n" % f)
            result = 2
        else:
            util.chmod(self.path,0755,self.m)
        #;

        for f in self.files:
            if not os.path.exists(f):
                self.m.warning("Not found: %s\n" % f)
                result = 1
            else:
                self.m.debug("OK %s\n" % f)
                #self.m.success(f+'\n')
                util.chmod(f,0644,self.m)
            #;
        #;

        for f in self.rpmdir.path, self.srpmdir.path:
            if not os.path.isdir(f):
                self.m.failure ("Not found: %s\n" % f)
                result = 2
            else:
                util.chmod(f,0755,self.m)
            #;
        #;

        # check my dirs
        for f in self.dirs:
            if not os.path.exists(f):
                self.m.warning("Not found: %s\n" % f)
                result = 1
            else:
                self.m.debug("OK %s\n" % f)
                #self.m.success(f+'\n')
                util.chmod(f,0755,self.m)
            #;
        #;

        # check symlinks
        for f in self.symlinks:
            if not os.path.exists(f):
                self.m.failure("Missing symlink: %s\n" % f)
                result = 2
            else:
                if os.path.islink(f):
                    self.m.success("symlink %s --> %s\n" % (f,os.path.realpath(f)))
                else:
                    self.m.failure ("Improper symlink %s\n" % f)
                    result = 2
                #;
            #;
        #;

        # check RPM and SRPM dirs
        for d in self.rpmdir, self.srpmdir:
            status = d.check(level)
            if status > 0:
                result = status
            #;
        #;

        if result == 0:
            self.m.success("Component %s\n" % self.id)
        elif result == 1:
            self.m.warning("in component %s\n" % self.id)
        elif result == 2:
            self.m.failure("in component %s\n" % self.id)
        #;
        return result
    #;

    def info(self):
        import string
        
        self.m.info("Component %s, path: %s\n" % (self.id, self.path))
        if self.local:
            s = "Access restricted, allowed hosts: "+string.join(self.allow,' ')+'\n'
            self.m.info(s)
        #;
        if self.ignore:
            self.m.warning("IGNORED\n")
        #;

        self.m.debug("Files owned: %s\n"%str(self.files))
        self.m.debug("Dirs owned: %s\n"%str(self.dirs))
        self.rpmdir.info()
        self.srpmdir.info()
    #;

    # Set up the datastructure for component, and if not existing, create
    # the necessary directories and files. This step allows us to get going
    # with mirroring.

    def create(self):

        if self.ignore: return
        if not (self.path and self.id):
            self.m.failure("must define path and id to create component\n")
            raise SystemExit
        #;

        # Make owned directories
        for d in self.dirs:
            if not os.path.exists(d):
                os.makedirs(d)
            #;
        #;

        # Make directories for rpms
        for d in self.rpmdir, self.srpmdir:
            d.create()
        #;
        
        # Make symlinks for yum
        if os.path.exists(self.yumdir):

            # The handling the symlinks below is complicated by the fact
            # that the user may have been moving files around so the
            # symlink exists but points to a non-existing directory.
            
            rpmlink = os.path.join(self.yumdir,"RPMS")
	    reldir = os.path.join("..", os.path.split(self.rpmdir.path)[1])
	    
	    if not os.path.exists(rpmlink):
                try:
                    os.symlink(reldir, rpmlink)
                except:
                    os.unlink(rpmlink)
                    os.symlink(reldir, rpmlink)
                #;
            else:
                if not os.path.islink(rpmlink):
                    self.m.warning("Not a symlink, deleting %s\n" % rpmlink)
                    try:
                        os.unlink(rpmlink)
                        os.symlink(reldir, rpmlink)
                    except OSError:
                        self.m.failure("Delete manually: %s\n" % rpmlink)
                    #;
                #;
            #;
            
            srpmlink = os.path.join(self.yumdir,"SRPMS")
	    sreldir = os.path.join("..", os.path.split(self.srpmdir.path)[1])
            if not os.path.exists(srpmlink):
                try:
                    os.symlink(sreldir, srpmlink)
                except:
                    os.unlink(srpmlink)
                    os.symlink(sreldir, srpmlink)
                #;
            else:
                if not os.path.islink(srpmlink):
                    self.m.warning("Not a symlink, deleting %s\n" % srpmlink)
                    try:
                        os.unlink(srpmlink)
                        os.symlink(sreldir, srpmlink)
                    except OSError:
                        self.m.failure("Delete manually: %s\n" % srpmlink)
                    #;
                #;
            #;
        #;

        if self.local:

            # If this component is local, we drop a few .htaccess files to
            # restrict access. It can inherit the local attribute from a
            # parent, but we only need to put the .htaccess file where
            # there are RPMs stored.

            util.create_htaccess(self.rpmdir.path,self.allow)
            util.create_htaccess(self.srpmdir.path,self.allow)
        #;

        # create the release.<id> file if non-existant
        ff = os.path.join(self.basedir,"release.%s" % self.id)
        if not os.path.exists(ff):
            f = open(ff,"w")
            f.write("Archive: Unknown\nComponent: "+self.id)
            f.write("\nVersion: Unknown\nOrigin: %s\nLabel: Unknown\n"%os.uname()[1])
            f.write("Architecture: %s\nNotAutomatic: false\n"%self.arch)
        #;

        # Create an rpmmacros file for the local build environment.
        # Don't overwrite it if it exists
        ff = os.path.join(self.basedir,"rpmmacros.%s" % self.id)
        if not os.path.exists(ff):
            util.create_rpmmacros(ff, os.path.join(self.rpmdir.path),
                                  os.path.join(self.srpmdir.path))
        #;
        return
    #;


    # Destroy the component dirs. If you're sick of your repository,
    # you can get rid of it here... :-{
    
    def destroy(self):
        if self.ignore: return
        if not self.path or not os.path.exists(self.path):
            return
        #;

        self.m.append (' '+self.id)

        for f in self.files:
            try:
                os.unlink(f)
            except OSError:
                pass
            #;

        for f in self.symlinks:
            try:
                os.unlink(f)
            except OSError:
                pass
            #;

        self.rpmdir.destroy()
        self.srpmdir.destroy()

        for f in self.dirs:
            try:
                os.rmdir(f)
            except OSError, e:
                self.m.failure("%s, %s\n"% (e.strerror,f))
            #;
        #;
    #;

    # Generate package lists etc. for the apt distribution.  This method is
    # a much simplified version of genbasedir, which is a pretty hairy
    # shell script.
    
    def generate (self):
        import md5, stat
        import shelve

        if self.ignore: return
        # genpkglist needs to run in the basedir
        os.chdir(self.basedir)
        os.nice(5)

        rsize = self.rpmdir.size()
        ssize = self.srpmdir.size()
        self.m.info("Generate pkglist for %s %s [%d files]\n" % (self.path, self.id, rsize+ssize))

        genpkglist = "/usr/bin/genpkglist"
        gensrclist = "/usr/bin/gensrclist"
        if not os.path.exists(genpkglist) or not os.path.exists(gensrclist):
            self.m.failure(genpkglist+" and/or "+gensrclist+" missing\n")
            return 2
        #;

        pkglist = os.path.join(self.basedir,"pkglist."+self.id)
        srclist = os.path.join(self.basedir,"srclist."+self.id)
        # index file for gensrclist (later)
        idxfile = os.path.join(self.basedir,"index."+self.id)

        if util.isnewer(self.rpmdir.path, pkglist+".bz2") > 0:
            # pkglist is sometimes not deleted???
            try:
                os.unlink(pkglist)
            except:
                pass
            #;
            L = [genpkglist,"--bloat", "--index", idxfile, self.path, self.id]
            os.spawnvpe(os.P_WAIT, "genpkglist", L, os.environ)
            self.m.success("pkglist created for component %s\n" % self.id)
        else:
            self.m.info("... pkglist was up-to-date for component %s\n" % self.id)
        #;
                    
        if util.isnewer(self.srpmdir.path, srclist+".bz2") > 0:
            # pkglist is sometimes not deleted???
            try:
                os.unlink(srclist)
            except:
                pass
            #;
            L = [gensrclist, "--flat", self.path, self.id, idxfile]
            if os.spawnvpe(os.P_WAIT, "gensrclist", L, os.environ) == 0:
                self.m.success("srclist created for component %s\n" % self.id)
            else:
                self.m.failure("creating srclist for component %s\n" % self.id)
            #;
        else:
            self.m.info("... srclist was up-to-date for component %s\n" % self.id)
        #;

        # Add the md5 signatures of the pkglists to shelve file md5.<comp>

        dict = shelve.open (os.path.join(self.basedir,"md5."+self.id),flag='c')

        for f in ["pkglist."+self.id, "srclist."+self.id,"release."+self.id]:
            # self.id is unicode => f is unicode
            if os.path.exists(f):
                md5calc = md5.new()
                fd = open(f)
                md5calc.update(fd.read())
                dict[str(f)] = ((md5calc.hexdigest(), fd.tell(), os.path.join("base", fd.name)))
                fd.close()
            #;
        dict.close()
        
        # bzip2 the file.
        L = ["bzip2", "--force", "-5", pkglist]
        if os.path.exists(pkglist):
            os.spawnvpe(os.P_NOWAIT, "bzip2", L, os.environ)
        #;

        # now bzip2 the srclist file
        if os.path.exists(srclist):
            L[-1] = srclist
            os.spawnvpe(os.P_NOWAIT, "bzip2", L, os.environ)
        #;

        # Now we make the YUM stuff. First the header.info and
        # header.src.info files...
        
        infofile = os.path.join(self.hdrdir,"header.info")
        if util.isnewer(self.rpmdir.path, infofile) > 0:
            fd = open(infofile, "w")
            for r in self.rpmdir.get_headers():
                ff = r[0]
                f = os.path.basename(ff)
                id = r[1].get_id(1)
                fd.write("%s=RPMS/%s\n" %(id,f))
            #;
            fd.close()
        else:
            self.m.info("... header.info was up-to-date for component %s\n" % self.id)
        #;

        infofile = os.path.join(self.hdrdir,"header.src.info")
        if util.isnewer(self.srpmdir.path, infofile) > 0:
            fd = open(infofile, "w")
            for r in self.srpmdir.get_headers():
                ff = r[0]
                f = os.path.basename(ff)
                id = r[1].get_id(1)
                fd.write("%s=SRPMS/%s\n" %(id,f))
            #;
            fd.close()
        else:
            self.m.info("... header.src.info was up-to-date for component %s\n" % self.id)
        #;

        # Finally, generate metadata for yum >= 2.2. AFAICS, this is generated
        # from scratch every time.
        util.generate_metadata(self.m, self.yumdir)

        return 0
    #;

    # Override the size() method from base class. Return a tuple of the number
    # of files in the RPMS and SRPMS directories, respectively...
    
    def size(self):
        if self.ignore: return (0,0)
        return (self.rpmdir.size(),self.srpmdir.size())
    #;

    # Clean out all regular files ending in .<comp>, leaving only
    # those owned by the class.

    def clean(self,delete):
        import glob

        if self.ignore: return []
        self.m.info("Cleaning component %s\n" % self.id)

        self.rpmdir.clean(delete)
        self.srpmdir.clean(delete)
        
        # clean out files in basedir... we have to be careful not
        # to delete files that belong to other components
        for d in self.path, self.basedir:
            L = glob.glob(os.path.join(d,"*."+self.id))
            files = map(lambda x: os.path.join(d,x), L)
            for f in files:
                #if f not in self.files and f not in self.dirs:
                if f not in self.files + self.dirs:
                    ff = os.path.join(self.path,f)
                    reason = "not owned by me (Component)"
                    if os.path.isdir(f):
                        what = "directory"
                    else:
                        what = "file"
                    #;
                    self.m.warning("convicted %s %s, %s\n" % (what,ff,reason))
                    self.death_row.append((ff,reason))
                #;
            #;
        #;

        # clean out yumdir and hdrdir. Here we simply get rid of everything
        # we don't know about...
        for d in self.yumdir, self.hdrdir:
            K = os.listdir(d)
            files = map(lambda x: os.path.join(d,x), K)
            for f in files:
                #if f not in self.files + self.dirs + self.symlinks + L:
                if f not in self.files + self.dirs + self.symlinks + self.hdrfiles:
                    ff = os.path.join(self.path,f)
                    reason = "not owned by me (Component)"
                    if os.path.isdir(f):
                        what = "directory"
                    else:
                        what = "file"
                    #;
                    self.m.warning("convicted %s %s, %s\n" % (what,ff,reason))
                    self.death_row.append((ff,reason))
                #;
            #;
        #;

        # Delete files on death row...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            self.shelve("death_row",self.death_row)
        #;
        return self.files+self.dirs
    #;

    # Method to create the html page for a component. It contains a
    # table of all RPMs that are part of this component.

    def webgen(self, Document, htmldir):
        import HTMLgen, time

        if self.ignore: return
        self.m.info("Component %s\n"% self.id)

        HTMLgen.PRINTECHO = 0
        doc = Document (self,stylesheet=self.repo.stylesheet)
        doc.header(self.repo.id)
        doc.append(HTMLgen.Href("index"+doc.suffix,"[Up]"))
        doc.append( HTMLgen.Heading(3, "%s %s [%s]" %
                                    (self.parent.parent.id, self.parent.id,self.arch)))
        if self.local:
            s = "<em>(restricted access)</em>"
        else:
            s = ""
        #;
        doc.append( HTMLgen.Heading(2, "Component %s %s" % (self.id, s)))
        if self.description:
            doc.append(self.description)
        #;
        if self.dfile:
            try:
                f = open(self.dfile)
                doc.append_subst(f.read())
                f.close()
            except IOError:
                pass
            #;
        #;

        doc.append ( "<br/>Contains %d RPMs and %d SRPMs<p>" % self.size())
        # Make table heading, body and caption #
        heading = HTMLgen.TR() + map(HTMLgen.TH,["Name", "Version", "Release",
                                                 "Arch", "Summary", "Source RPM",
                                                 "Buildtime"])

        # Make the table
        table = HTMLgen.TableLite(Class="table")
        table.append(heading)
        doc.append(table)

        j = 0
        for f,h in self.rpmdir.get_headers():
            t = time.strftime("%d-%b-%Y %T",time.localtime(h["BUILDTIME"]))

            relpath = os.path.join("..","RPMS."+self.id, os.path.basename(f))
            relsrcpath = os.path.join("..","SRPMS."+self.id, h["SOURCERPM"])
            srcpath = os.path.join(self.srpmdir.path, h["SOURCERPM"])

            # create a link to the src.rpm if available
            u = HTMLgen.Href(relpath ,h["NAME"])
            if os.path.exists(srcpath):
                s = HTMLgen.Href(relsrcpath , "Yes")
            else:
                s = "No"
            #;

            # Some rogue characters make htmlgen croak. Actually, the rpm
            # interface is in error, because the description returns an
            # ascii string, even if it contains characters >127.

            summary = HTMLgen.Href(os.path.basename(f)+doc.suffix,
                                   HTMLgen.RawText(util.fixrogue(h["SUMMARY"])))
            tdlist = map(HTMLgen.TD,[u, h["VERSION"],h["RELEASE"],
                                     h["ARCH"],summary, s, t])
            if j%2 == 0:
                col = "col1"
            else:
                col = "col2"
            #;
            row = HTMLgen.TR(Class=col)
            row = row + tdlist
            table.append(row)
            j = j+1
        #;

        doc.footer()
        doc.write(os.path.join(htmldir,"index_"+self.id +doc.suffix))
        self.rpmdir.repo = self.repo
        self.rpmdir.webgen(Document,htmldir)
    #;


    # Build all the src.rpm's. This implementation is very simple, I will
    # replace it with an interface to mach sometime.

    def build(self):

        if self.ignore: return
        self.m.info("Build component %s\n"%self.id)
        for fullpath in self.srpmdir:
            f=os.path.basename(fullpath)
            if not f in self.rpmdir:
                self.m.info("Gonna rebuild %s\n"%f)
                rpmmacros = os.path.join(self.basedir,"rpmmacros.%s" % self.id)
                L = ["rpmbuild", "--rebuild", "--macros", rpmmacros, fullpath]
                if os.spawnvpe(os.P_WAIT, "rpmbuild", L, os.environ) == 0:
                    self.m.success("Rebuild of %s succeeded\n" % f)
                else:
                    self.m.failure("Rebuild of %s\n" % f)
                #;
            #;
        #;
    #;


    # Audit is an extensive check on the index and header files,
    # md5 sums and more...

    def audit(self,delete):
        import shelve, rpmfile2

        # Make sure we have an up-to-date rpm list
        for d in self.children:
            d.audit()
        #;

        hdrdict = shelve.open(self.rpmdir.hdrdictpath)

        # Check all rpm files. Make sure an entry exists in hdrdict,
        # make sure that the header cache exists.
        
        for f in self.rpmdir:
            if not os.path.exists(f):
                self.rpmdir.remove(f)
                if hdrdict.has_key(f):
                    del hdrdict[f]
                #;
                self.m.debug("entry %s removed from file list\n", f)
            #;
        #;
        hdrdict.sync()

        for f in self.rpmdir.data + self.srpmdir.data:
            if isinstance(f, unicode):
                f = f.encode("utf-8")
            #;
            if not hdrdict.has_key(f):
                
                # A file is found in RPMS.<comp> or SRPMS.<comp> which does
                # not have an entry in hdrdict.db

                self.m.debug("not in hdrdict.db %s\n"% f)
                r = rpmfile2.RPMfile(f)
                try:
                    r.open()
                    hdrfile = os.path.join(self.hdrdir,r.get_id()+".hdr")
                    hdrdict[f] = hdrfile
                    if isinstance(hdrfile, unicode):
                        hdrfile = hdrfile.encode("utf-8")
                    hdrdict[hdrfile] = f
                    r.write(hdrfile)
                    self.m.debug("header cached in %s\n" % hdrfile)
                except rpmfile2.RPMerror:
                    self.m.failure("problem with file %s\n" % f)
                    self.death_row.append((f,"rpm error"))
                #;
            #;
        #;

        # Check that all keys in hdrdict map to an existing file...
        
        for f in hdrdict.keys():
            if not os.path.exists(f):
                del hdrdict[f]
                self.m.debug("deleted orphaned hdrdict entry: %s\n" % f)
            #;
        #;
        hdrdict.sync()


        # Check that all .hdr files have an entry in hdrdict, if not
        # just delete it (it can easily be re-created)

        for f in self.hdrfiles:
            if isinstance(f, unicode):
                f = f.encode("utf-8")
            #;
            if not hdrdict.has_key(f) or not os.path.exists(hdrdict[f]):
                reason = "orphaned hdr file"
                self.m.warning("convicted file %s, %s\n" % (f,reason))
                self.death_row.append((f,reason))
            #;
        #;

        # Delete files on death row...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            self.shelve("death_row", self.death_row)
        #;

        hdrdict.close()
        return
    #;

#; end of class Component

####
