# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: distribution.py,v 1.21 2004/10/08 13:33:19 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import os
from element import Element
import util, message

class Distribution(Element):

    m = message.Message()
    m.set_indent(1)

    def __init__(self):
        Element.__init__(self)
        self.type = 1
    #;

    # Override set_path from base class "Element", because we need
    # to do some extra stuff
    def set_path(self, path):
        Element.set_path(self,path)
        self.dirs.append(path)
        return
    #;

    def info(self):

        self.m.info("Distribution %s, path: %s\n" % (self.id, self.path))
        self.m.debug("Files owned: %s\n"%str(self.files))
        self.m.debug("Dirs owned: %s\n"%str(self.dirs))
        if self.ignore:
            self.m.warning("IGNORED\n")
        #;
        for rel in self.children:
            rel.info()
        #;

    # Initiate a complete check on the distribution. We check for
    # presence of files and protection modes
    
    def check(self, level):

        if self.ignore: return 0

        self.m.info("Checking distribution %s\n" % self.id)
        result = 0

        f = self.path
        if not os.path.isdir(f):
            self.m.failure("Distribution directory not found: %s\n" % f)
            result = 2
        else:
            util.chmod(f,0755,self.m)
        #;

        for rel in self.children:
            stat = rel.check(level)
            if stat > 0:
                result = stat
            #;
        #;

        if result == 0:
            self.m.success("Distribution %s\n" % self.id)
        elif result == 1:
            self.m.warning("in distribution %s\n" % self.id)
        elif result == 2:
            self.m.failure("in distribution %s\n" % self.id)
        #;
        return result
    #;

    # Create the distribution, and ask the children to go create
    # themselves... hmm, sounds worse than it is ;-)

    def create(self):

        if self.ignore: return
        
        if self.path and not os.path.exists(self.path):
            self.m.info("Creating distribution %s\n" % self.id)
            # if path is set and does not exist try
            try:
                os.mkdir(self.path)
            except OSError:
                # topdir does not exist
                self.m.failure("could not create directory %s\n" % self.path)
                self.m.info ("topdir probably doesn't exist!\n")
                raise SystemExit
            #;
        #;
        for rel in self.children:
            rel.create()
        #;
    #;

    # Destroy the entire distribution
    
    def destroy(self):

        if self.ignore: return
        
        if not self.path or not os.path.exists(self.path):
            return
        #;

        self.m.warning("Destroying distribution %s\n" % self.id)

        for rel in self.children:
            rel.destroy()
        #;
        os.rmdir(self.path)
    #;


    # Delete all files not known by me or my children. Return
    # a list of files owned by me, so parent does not delete them

    def clean(self,delete):
        if self.ignore: return [self.files+self.dirs]
        
        self.m.info("Cleaning distribution %s\n" % self.id)
        L = []
        for rel in self.children:
            L = L+rel.clean(delete)
        #;

        K = os.listdir(self.path)
        files = map(lambda x: os.path.join(self.path,x), K)
        for f in files:
            if f not in self.files and f not in self.dirs and f not in L:
                reason = "not owned by me (Distribution)"
                if os.path.isdir(f):
                    what = "directory"
                else:
                    what = "file"
                #;
                self.m.warning("convicted %s %s, %s\n" % (what,f,reason))
                self.death_row.append((f,reason))
            #;
        #;

        # Delete files on death row...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            self.shelve("death_row", self.death_row)
        #;
        return self.files+self.dirs
    #;

    # Generate webpages for this distribution. In fact, just pass
    # on the job to the children...
    
    def webgen(self,Document):

        if self.ignore: return [(None,0,0,0)]
        self.m.info("Web pages for distribution %s\n" % self.id)

        L = []
        for rel in self.children:
            if not rel.ignore:
                rel.webgen(Document)
                L.append ((self.id, rel.id, rel.arch, rel.size()))
            #;
        #;
        return L
    #;
#;

####
