# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: element.py,v 1.15 2004/10/18 16:18:38 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA


# Base class for the hierachy of classes describing a repo. Make it a
# new style object...

import os, shelve
import util

class Element(object):

    def __init__(self):
        self.path = None                # absolute path, for most ops
        self.relpath = None             # relative path, for urls
        self.id = None                  # my name
        self.arch = None                # my arch
        self.local = False              # if only local access
        self.allow = []                 # these hosts are allowed access
        self.files = []                 # these files are owned by the class
        self.children = []              # these are the children of the class
        self.death_row = []             # these files are doomed...
        self.dirs = []                  # these dirs are owned by class
        self.description = ''           # non-html string description
        self.dfile = None               # name of description file
        self.parent = None              # this is my parent
        self.ignore = False             # if set, ignore me
        self.shelf = None               # name of shelf
    #;

    def set_path(self, path):
        self.path = path
        if self.id:
            # Repo has a descriptive self.id, so we deal with that
            if self.type > 0:
                self.shelf = os.path.join(self.path,".data_"+self.id+".db")
            else:
                self.shelf = os.path.join(self.path,".data_repo.db")
            #;

            self.files.append(self.shelf)
        #;
        return
    #;

    def set_id(self, id):
        self.id = id
        if self.path:
            self.shelf = os.path.join(self.path,".data_"+self.id+".db")
            self.files.append(self.shelf)
        #;
        return
    #;

    def set_local(self,local):
        self.local = local
    #;

    def set_parent(self,p):
        self.parent = p
    #;
    
    # Add a child to the element
    def add(self, child):
        self.children.append(child)
    #;

    # Save some object on my shelf
    def shelve(self, name, obj):
        if obj:
            D = shelve.open(self.shelf)
            D[name] = obj
            D.close()
        #;
    #;

    # Ask children to create themselves
    
    def create(self):

        if self.ignore: return
        what =  self.__class__.__name__
        self.m.info ("Create for %s %s\n" %(what, self.id))

        for d in self.children:
            d.create()
        #;
        return
    #;

    # Ask children to go get data from the mirror sites
    
    def mirror(self):

        if self.ignore: return
        what =  self.__class__.__name__
        self.m.info ("Mirroring %s %s (%s)\n" %(what, self.id, self.path))
        for dist in self.children:
            dist.mirror()
        #;
        return
    #;

    # Generate all indeces etc. required for the repo to function

    def generate(self):

        if self.ignore: return 0

        what =  self.__class__.__name__
        self.m.info ("Generate for %s %s\n" %(what, self.id))

        result = 0
        for rel in self.children:
            res = rel.generate()
            if res > result:
                result = res
            #;
        #;
        return result
    #;
    
    # Return a tuple of the number of files in the RPMS and SRPMS
    # in the entire distribution

    def size(self):
        if self.ignore: return(0,0)
        rsize = ssize = 0
        for rel in self.children:
            r,s = rel.size()
            rsize = rsize + r
            ssize = ssize + s
        #;
        return(rsize,ssize)
    #;


    def purge(self,delete):
        if self.ignore: return

        what =  self.__class__.__name__
        self.m.info ("Purge of %s %s\n" %(what, self.id))

        for dist in self.children:
            dist.purge(delete)
        #;
        return
    #;


    # Ask children to build RPMs from SRPMs.
    def build(self):
        if self.ignore: return

        what =  self.__class__.__name__
        self.m.info ("Build in %s %s\n" %(what, self.id))

        for d in self.children:
            d.build()
        #;
        return

    # Audit is a very rigorous checking
    def audit(self, delete):
        if self.ignore: return

        what =  self.__class__.__name__
        self.m.info ("Audit of %s %s\n" %(what, self.id))

        result = 0
        for d in self.children:
            res = d.audit(delete)
            if res > result:
                result = res
        #;
        if result == 0:
            self.m.success("%s %s\n" % (what,self.id))
        elif result == 1:
            self.m.warning("%s %s\n" % (what,self.id))
        elif result == 2:
            self.m.failure("%s %s\n" % (what,self.id))

        return result
    #;

    # Read the death_row data from the shelf and delete the files in it
    def zap(self):
        if self.ignore: return 0
        
        result = 0
        what =  self.__class__.__name__

        self.m.info ("Zap convicted files from %s %s\n" %(what, self.id))
        D = shelve.open(self.shelf)
        if D.has_key("death_row"):
            death_row = D.get("death_row")
            util.delete(self.m, death_row)
            del D["death_row"]
        #;
        D.close()
        
        for d in self.children:
            res = d.zap()
            if res > result:
                result = res
        #;
        if result == 0:
            self.m.success("%s %s\n" % (what,self.id))
        elif result == 1:
            self.m.warning("%s %s\n" % (what,self.id))
        elif result == 2:
            self.m.failure("%s %s\n" % (what,self.id))

        return result
    #;
#;

####
