# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: release.py,v 1.33 2004/12/07 19:33:06 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# A release consists of a number of components and an architechture.

import os, sys
from element import Element
import component
import util, message

class Release(Element):

    m = message.Message()
    m.set_indent(2)

    def __init__(self):
        Element.__init__(self)
        self.basedir = None
        self.htmldir = None
        self.type = 2
    #;

    # Method to set up list of files and directories owned by this
    # class. It is called once, when both path and arch have been set.
    def make_files_list(self):
        self.basedir = os.path.join(self.path,self.arch,"base")
        self.htmldir = os.path.join(self.path,self.arch,"html")

        # these dirs are owned by me
        self.dirs.append(self.basedir)
        self.dirs.append(self.htmldir)
        self.dirs.append(self.path)
        self.dirs.append(os.path.join(self.path,self.arch))
        
        # these files are owned by me
        ff = os.path.join(self.basedir,"release")
        self.files.append(ff)
        ff = os.path.join(self.path,self.arch,"html","index.html")
        self.files.append(ff)
    #;


    # Set the architecture, and also set the basedir (either here or
    # in set_path() later on; we don't know which one will be called
    # first)
    def set_arch(self, arch):
        self.arch = arch
        if self.path and not self.basedir:
            self.make_files_list()
        #;
    #;

    # Override the base class method, since we need to set the basedir
    # (either here or in set_arch() later on; we don't know which one
    # will be called first)
    def set_path(self, path):
        Element.set_path(self,path)
        if self.arch and not self.basedir:
            self.make_files_list()
        #;
    #;

    # Run a check on the existence of directories owned by this
    # class
    def check(self,level):

        if self.ignore: return
        
        self.m.info("Checking release %s arch %s\n" % (self.id, self.arch))
        result = 0

        f = os.path.join(self.path,self.arch)
        if not os.path.isdir(f):
            self.m.failure("Release directory not found: %s\n" % f)
            result = 2
        else:
            util.chmod(f,0755,self.m)
        #;

        if not os.path.isdir(self.basedir):
            self.m.failure("No base directory for release %s\n" % f)
            result = 2
        else:
            util.chmod(self.basedir,0755,self.m)
        #;

        if not os.path.isdir(self.htmldir):
            self.m.failure("No html directory for release %s\n" % f)
            result = 2
        else:
            util.chmod(self.htmldir,0755,self.m)
        #;

        # Fix permissions of html files
        for f in os.listdir(self.htmldir):
            util.chmod(os.path.join(self.htmldir,f),0644,self.m)
        #;

        for f in self.files:
            if not os.path.exists(f):
                self.m.warning("Not found: %s\n" % f)
            else:
                self.m.success(f+'\n')
                util.chmod(f,0644,self.m)
            #;
        #;

        for d in self.dirs:
            if not os.path.exists(d):
                self.m.warning("Dir not found: %s\n" % d)
            else:
                self.m.success(d+'\n')
                util.chmod(d,0755,self.m)
            #;

        if result == 0:
            for c in self.children:
                stat = c.check(level)
                if stat > 0:
                    result = stat
                #;
            #;
        #;
        if result == 0:
            self.m.success("Release %s\n" % self.id)
        elif result == 1:
            self.m.warning("in release %s\n" % self.id)
        elif result == 2:
            self.m.failure("in release %s\n" % self.id)
        #;
        return result
    #;

    # Return some info on the Release level
    def info(self):

        self.m.info("Release %s %s [%s]\n" % (self.id,self.path,self.arch))
        self.m.debug("Files owned: %s\n"%str(self.files))
        self.m.debug("Dirs owned: %s\n"%str(self.dirs))
        if self.ignore:
            self.m.warning("IGNORED\n")
        #;
        for comp in self.children:
            comp.info()
        #;
    #;


    def create(self):
        import time

        if self.ignore: return
        print_flag = False
        
        if not (self.id and self.path):
            self.m.failure("must define path and arch to create release\n")
            raise SystemExit
        #;

        if self.path and not os.path.exists(self.path):
            self.m.info("Creating release %s [%s] " % (self.id, self.arch))
            os.mkdir(self.path)
            print_flag = True
        #;
        
        ff = os.path.join(self.path,self.arch)
        if not os.path.exists(ff):
            os.mkdir(ff)

        if not os.path.exists(self.basedir):
            os.mkdir(self.basedir)
        #;

        if not os.path.exists(self.htmldir):
            os.mkdir(self.htmldir)
        #;

        for comp in self.children:
            comp.create()
            if print_flag:
                self.m.append(' '+comp.id)
            #;
        #;
        if print_flag:
            self.m.append('\n')
        #;

        # create a release file in the base dir
        ff = os.path.join(self.basedir,"release")
        self.files.append(ff)
        if not os.path.exists(ff):
            f = open(os.path.join(self.basedir,"release"), "w")
            util.create_release(f,self.arch, self.children)
            f.close()
        #;
    #;

    # Destroy the release
    def destroy(self):
        import sys

        if self.ignore: return

        if not self.path or not os.path.exists(self.path):
            return
        #;
        
        self.m.warning("Destroying release %s " % self.id)
        for comp in self.children:
            comp.destroy()
        #;
        self.m.append ('\n')
        
        for f in self.files:
            try:
                os.unlink(f)
            except OSError:
                pass
            #;
        #;
        os.rmdir(self.basedir)
        os.rmdir(self.htmldir)
        os.rmdir(os.path.join(self.path,self.arch))
        os.rmdir(os.path.join(self.path))
        #;
    #;


    # Override base class method. Generate indeces for this release.

    def generate(self):
        import shelve
        
        if self.ignore: return
        
        self.m.info("Release %s\n" % self.id)

        # comps will be computing new md5 info so create
        # a fresh new file.
        f = open(os.path.join(self.basedir,"release"), "w")
        util.create_release(f, self.arch, self.children)

        result = 0
        for comp in self.children:
            res = comp.generate()
            if res > result:
                result = res
            #;
        #;

        for comp in self.children:
            d = shelve.open(os.path.join(comp.basedir,"md5."+comp.id))
            for i in d.keys():
                f.write(" %s %d %s\n" % d[i])
            #;
            d.close()
        #;
        f.close()
        return result
    #;


    # Delete all files not known by me or my children. Return
    # a list of files owned by me, so parent does not delete them

    def clean(self,delete):

        if self.ignore: return [self.path]

        self.m.info("Cleaning release %s\n" % self.id)
        L = []
        for comp in self.children:
            L = L+comp.clean(delete)
        #;

        for d in self.basedir, os.path.join(self.path,self.arch):
            K = os.listdir(d)
            files = map(lambda x: os.path.join(d,x), K)
            for f in files:
                if f not in self.files + self.dirs + L:
                    ff = os.path.join(self.path,f)
                    reason = "not owned by me (Release)"
                    if os.path.isdir(f):
                        what = "directory"
                    else:
                        what = "file"
                    #;
                    self.m.warning("convicted %s %s, %s\n" % (what,ff,reason))
                    self.death_row.append((ff,reason))
                #;
            #;
        #;

        # Delete files on death row...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            self.shelve("death_row", self.death_row)
        #;
        return self.files+self.dirs
    #;


    # The release class is respansible for generating the HTML pages.
    # It simply creates links to the individual component pages.

    def webgen(self, Document):
        import HTMLgen
        import stat, time, string

        if self.ignore: return
        
        self.m.info("Generate web pages for release %s [%s]\n" % (self.id, self.arch))

        HTMLgen.PRINTECHO = 0
        doc = Document (self,stylesheet=self.repo.stylesheet)
        doc.header(self.repo.id)
        doc.append(HTMLgen.Href(os.path.join("..","..","..","..","index"+doc.suffix),"[Up]"))
        doc.append( HTMLgen.Heading(2, "Distribution %s %s" % (self.parent.id,self.id)))

        if self.description:
            doc.append(self.description)
        #;
        if self.dfile:
            try:
                f = open(self.dfile)
                doc.append_subst(f.read())
                f.close()
            except IOError:
                pass
            #;
        #;

        doc.append ( "<br/>Contains %d RPMs and %d SRPMs<p>" % self.size())

        heading = HTMLgen.TR() + map(HTMLgen.TH,["Component", "Arch", "Summary", "RPMs", "SRPMs", "Last update"])
        table = HTMLgen.TableLite(Class="table")
        table.append(heading)

        rpmslist = []
        srpmslist = []
        got1 = got2 = False

        j = 0
        for comp in self.children:
            siz = comp.size()
            st = os.stat(comp.rpmdir.path)
            age = time.strftime("%d %b %Y %T",time.localtime(st[stat.ST_MTIME]))
            href = HTMLgen.Href("index_"+comp.id+doc.suffix, comp.id)
            tdlist = map(HTMLgen.TD,[href,comp.parent.arch,comp.description, siz[0], siz[1], age])
            if j%2 == 0:
                col = "col1"
            else:
                col = "col2"
            #;
            row = HTMLgen.TR(Class=col)
            row = row + tdlist
            table.append(row)
            j = j+1
            if siz[0] > 0:
                rpmslist.append(comp.id)
                got1 = True
            #;
            if siz[1] > 0:
                srpmslist.append(comp.id)                
                got2 = True
            #;
        #;

        # Output the table of components...
        doc.append(table)

        # Now create lines telling viewer how to customize apt...
        if got1 or got2:

            doc.append("<p>For apt, put the following line(s) in a file in directory")
            doc.append ("<code>/etc/apt/sources.list.d/</code>:<p>")
            table = HTMLgen.TableLite(Class="listing")

            if got1:
                row = HTMLgen.TR(Class="listing")
                tdlist = map(HTMLgen.TD, ["rpm", self.repo.url,
                                          os.path.join(self.parent.id, self.id, self.arch),
                                          string.join(rpmslist,' ')])
                row = row + tdlist
                table.append(row)
            #;
            if got2:
                row = HTMLgen.TR(Class="listing")
                tdlist = map(HTMLgen.TD, ["rpm-src", self.repo.url,
                                          os.path.join(self.parent.id, self.id, self.arch),
                                          string.join(srpmslist,' ')])
                row = row + tdlist
                table.append(row)
            doc.append(table)
        #;

        # ... and yum...

        if got1 or got2:

            doc.append("<p>For Yum, put the following lines in /etc/yum.conf:<p>")
            table = HTMLgen.TableLite(Class="listing")

            for comp in rpmslist:
                row = HTMLgen.TR(Class="listing")
                tdlist = map(HTMLgen.TD, ["[%s]"%comp])
                row = row + tdlist
                table.append(row)                

                row = HTMLgen.TR(Class="listing")
                tdlist = map(HTMLgen.TD, ["name=%s %s $releasever - $basearch - %s"%(self.repo.id,self.parent.id,comp)])
                row = row + tdlist
                table.append(row)

                row = HTMLgen.TR(Class="listing")
                tdlist = map(HTMLgen.TD, ["baseurl=%s/%s/$releasever/$basearch/%s" %(self.repo.url,self.parent.id,comp)])
                row = row+tdlist
                table.append(row)
            doc.append(table)
            doc.append("<p>You only need to include the components you are interested in.")
            
        #;

        doc.footer()
        doc.write(os.path.join(self.htmldir,"index"+doc.suffix))

        # Let the components generate their webpages...
        for comp in self.children:
            comp.webgen(Document,self.htmldir)
        #;

        return
    #;

##     # Build all SRPMs in this release.

##     def build(self):
##         if self.ignore: return
##         for comp in self.children:
##             comp.build()
##         #;
##         return
##    #;
#;

####
