# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: repository.py,v 1.14 2004/12/07 19:36:33 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If
# not, visit their website at http://www.gnu.org for a copy, or
# request a copy by writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import os, string, stat
from element import Element
import util, message


# Check whether the user has write permission to apt's cache (if it
# exists at all!). If not, issue a warning, since execution times can
# be up to 10x longer...

def checkapt(msg):
    # 1. Find group of:
    # "/var/cache/apt/pkgcache.bin" (?)
    # "/var/cache/apt/srcpkgcache.bin" (?)

    err_ex = False
    err_perm = False
    err_gr = False
    
    L = [ "/var/cache/apt/genpkglist", "/var/cache/apt/gensrclist"]
    for d in L:
        if not os.path.exists(d):
            err_ex = True
            continue
        #;
        st = os.stat(d)
        gr = st[stat.ST_GID]
        m = st[stat.ST_MODE]
        if (stat.S_IMODE(m) & stat.S_IWGRP) == 0:
            err_perm = True
        #;

        # 2. Is the user a member of the group that owns these directories?
        if gr not in os.getgroups():
            err_gr = True
        #;
    #;
    if err_ex or err_perm or err_gr:
        if not err_ex:
            msg.warning("md5 cache is not group writable, generate will be s-l-o-w!!\n")
            if err_perm:
                msg.info("action: as root, fix permissions:\n")
                msg.info("chmod g+rws %s\n"%string.join(L,' '))
            #;
            if err_gr:
                msg.info("action: make yourself a member of group %d\n" %gr)
            #;
        else:
            msg.warning("directories %s don't exist, apt is not installed properly\n"%str(L))
        #;
    else:
        msg.success("md5 cache is writable\n")
        return 0
    #;
    
    return 1
#;


class Repository (Element):

    maintainer = "Anonymous Coward"
    email = ""
    url = "http://localhost.localdomain"
    stylesheet = "rj.css"
    description = None
    m = message.Message()
    m.set_indent(0)
    rj = util.antispam("http://www.bioxray.dk/~mok/repo-janitor.php")

    def __init__(self):
        Element.__init__(self)
        self.children = []
        self.type = 0
        self.repo = self
        return
    #;

    # Override set_path from base class "Element", because we need
    # to do some extra stuff
    def set_path(self, path):
        Element.set_path(self,path)
        self.files.append(os.path.join(path,"index.html"))
        self.files.append(os.path.join(path,"rj.css"))
        return
    #;


    def info(self):
        self.m.info("%s, path: %s\n" % (self.id, self.path))
        self.m.info("Maintainer: %s, email: %s\n" % (self.maintainer, self.email))
        self.m.info("Repository URL: %s\n" % self.url)
        self.m.info("Stylesheet: %s\n" % self.stylesheet)
        self.m.debug("Files owned: %s\n" % str(self.files))        
        self.m.debug("Dirs owned: %s\n" % str(self.dirs))        
        if self.ignore:
            self.m.warning("IGNORED\n")
        #;
        for dist in self.children:
            dist.info()
        #;
        return
    #;


    # Initiate a complete check on the repository. We check for
    # presence of files and protection modes
    
    def check(self, level):

        if self.ignore: return
        self.m.info("Checking %s\n" % self.id)
        result = 0

        result = checkapt(self.m)

        f = self.path
        if not os.path.isdir(f):
            self.m.failure("topdir not found: %s\n" % f)
            result = 2
        else:
            util.chmod(f,0755,self.m)
        #;

        for f in self.files:
            if not os.path.exists(f):
                self.m.warning("Not found: %s\n" % f)
                result = 1
            else:
                self.m.success(f+'\n')
                util.chmod(f,0644,self.m)
            #;
        #;
            

        for dist in self.children:
            stat = dist.check(level)
            if stat > 0:
                result = stat
            #;
        #;

        if result == 0:
            self.m.success("in %s\n" % self.id)
        elif result == 1:
            self.m.warning("in %s\n" % self.id)
        elif result == 2:
            self.m.failure("in %s\n" % self.id)
        #;
        return result
    #;


    # Destroy the entire repository
    
    def destroy(self):

        if self.ignore: return
        if not self.path or not os.path.exists(self.path):
            return
        #;

        self.m.warning("Destroying %s\n" % self.id)

        for dist in self.children:
            dist.destroy()
        #;

        # Delete my files...
        for f in self.files:
            try:
                os.unlink(f)
            except OSError:
                pass
            #;
        return
    #;

    
    def clean(self,delete):

        if self.ignore: return self.files+self.dirs
        
        self.m.info("Cleaning %s\n" % self.id)
        L = []
        for dist in self.children:
            L = L+dist.clean(delete)
        #;
        K = os.listdir(self.path)
        files = map(lambda x: os.path.join(self.path,x), K)
        for f in files:
            if f not in self.files and f not in self.dirs and f not in L:
                reason = "not owned by me (Repository)"
                if os.path.isdir(f):
                    what = "directory"
                else:
                    what = "file"
                #;
                self.m.warning("convicted %s %s, %s\n" % (what,f,reason))
                self.death_row.append((f,reason))
            #;
        #;

        # Delete files on death row...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            self.shelve("death_row", self.death_row)
        #;
        return self.files+self.dirs
    #;

    # Produce main html page.
    
    def webgen(self,Document):
        import HTMLgen, time

        if self.ignore: return
        self.m.info("Web pages for %s\n" % self.id)
        L = []
        for dist in self.children:
            if not dist.ignore:
                L.append(dist.webgen(Document))
            #;
        #;

        doc = Document (self,stylesheet=self.repo.stylesheet)
        HTMLgen.PRINTECHO = 0

        doc.header(self.id)
        doc.append( HTMLgen.Heading(2, "Main page" ))
        if self.description:
            doc.append(self.description)
        #;
        if self.dfile:
            try:
                f = open(self.dfile)
                doc.append_subst(f.read())
                f.close()
            except IOError:
                pass
            #;
        #;

        if len(L) == 1:
            doc.append("<br/>1 distribution is available:")
        else:
            doc.append("<br/>%d distributions are available:" % len(L))
        ul = HTMLgen.OrderedList()
        for l in L:
            # l kan be empty if all releases in a distribution are ignored
            if l:
                #print l[0]
                ul.append("Distribution " + l[0][0])
                ul2 = HTMLgen.List()
                ul.append(ul2)
                for k in l:
                    htmldir = os.path.join(l[0][0],k[1],k[2],"html")
                    s = "Release %s, arch %s " % (k[1], k[2])
                    s = s + "[%d RPMs; %d SRPMs]" % k[3]
                    href = HTMLgen.Href(os.path.join(htmldir,"index"+doc.suffix),s)
                    ul2.append(href)
                #;
                ul2.end_element()
            #;
        #;
        ul.end_element()
        doc.append(ul)
        doc.footer()
        doc.write(os.path.join(self.path,"index"+doc.suffix))
        return 0
    #;

#;

####
