# -*- mode: python; mode: font-lock; py-indent-offset: 4; -*-
# $Id: rpmdir.py,v 1.48 2004/12/07 19:40:22 mok Exp $
#
# Copyright (C) 2004 Morten Kjeldgaard <mok@bioxray.dk>
#
# This file is part of the program "repo-janitor".
#
# "repo-janitor" is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# "repo-janitor" is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with "repo-janitor" in a file named "COPYING".  If not, visit
# their website at http://www.gnu.org for a copy, or request a copy by
# writing to:
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

import UserList
import os, string
import glob
import commands
import shutil
import util, message
from element import Element
import rpmfile2
import shelve

# This class describes a directory a set of RPMs,
# including an external URL to a mirror.

class RPMdir(UserList.UserList,Element):

    m = message.Message()
    m.set_indent(4)

    def __init__(self, path):
        Element.__init__(self)
        self.type = 4
        self.path = path
        self.data = []
        self.mirrors = []                 # specify mirrors to mirror from
        self.hdrs = []
        self.allowed_archs = []
        self.issource = False
 
        if os.path.exists(self.path):
            self.data = [] # a list of RPMfile classes
            for f in  glob.glob(os.path.join(self.path, "*.rpm")):
                self.data.append(f)
            #;
            self.data.sort()
            Element.set_id(self,os.path.basename(self.path))
        #;
        self.hdrdictpath = os.path.join(self.path,"hdrdict.db")
        self.files.append(self.hdrdictpath)

        self.md5dictpath = os.path.join(self.path,"md5dict.db")
        self.files.append(self.md5dictpath)
        return
    #;


    def set_local(self,local):
        self.files.append(os.path.join(self.path,".htaccess"))
        return
    #;

    # Print some information on this class

    def info(self):
        if os.path.exists(self.path):
            self.m.info("%d rpm files in %s\n" %(len(self.data),self.path))
            if self.mirrors:
                for u in self.mirrors:
                    self.m.info("mirror %s \n" % u)
                    u.print_cmd()
            #;
            self.m.info("allowed archs: %s\n" % string.join(self.archs,' '))
        #;
        self.m.debug("Files owned: %s %s\n"%(str(self.files),str(self.data)))
        return
    #;

    # Here we simply check to see if the files is there, and that they
    # have the right permissions

    def check(self,level):
        result = 0

        # It's ok to have an empty directory
        if not self.data:
            return result
        #;

        # Check the RPMs we think are there are there :-)
        for f in self.data:
            ff = os.path.join(self.path,f)
            if not os.path.exists(ff):
                self.m.failure("Not found %s\n" % ff)
                result = 2
                self.data.remove(f)
            else:
                util.chmod(ff,0644,self.m)
            #;
        #;

        # Check the files that should be there
        for f in self.files:
            ff = os.path.join(self.path,f)
            if not os.path.exists(ff):
                self.m.failure("Not found %s\n" % ff)
                result = 2
            else:
                util.chmod(ff,0644,self.m)
            #;
        #;

        # Check directories...
        for d in self.dirs:
            if not os.path.exists(d):
                self.m.failure("Not found %s\n" % d)
                result = 2
            else:
                util.chmod(d,0755,self.m)
            #;
        #;

        if level > 0:
            # We check the md5 sum of all files.
            pass
        #;
        return result
    #;

    # The string representation of the class is simply the path
    def __str__(self):
        return str(self.path)
    #;

    # Set up the datastructure for an rpmdir, and if they don't exist,
    # create the necessary directories and files.
    def create (self):

        if not os.path.exists(self.path):
            os.mkdir(self.path)
        #;

        if os.path.exists(self.path) and not self.data:
            self.data = [] # a list of RPMfile classes
            for f in  glob.glob(os.path.join(self.path, "*.rpm")):
                file = os.path.basename(f)
                self.data.append(file)
            #;
        #;
        return
    #;

    # This method takes care of mirroring the present directory
    # from some source, which may be web site or a local directory.

    def mirror(self):
        import urlparse, time

        if not self.mirrors:
            return
        #;

        # Create a lock file containing process id and timestamp
        lockf = os.path.join(self.path,"mirror_lock")
        str = "pid %d start %s" % (os.getpid(),time.ctime())
        if not util.lock(lockf, str):
            fd = open(lockf)
            msg = fd.read()
            fd.close()
            self.m.failure("Lock set by another process (started %s)\n" % msg)
            return
        #;

        self.m.info("Update %s from mirror\n" % self.id)
        
        for mirror in self.mirrors:
            self.m.info("Getting updates from mirror %s\n" % mirror)
            status = mirror.exe()
            if not status:
                self.m.failure("mirror %s\n" % mirror)
            else:
                self.shelve("last_mirrored", time.ctime())
            #;
        #;

        util.unlock(lockf)
        return
    #;

    # Return list of RPM headers. This can take long to generate for large
    # directories so don't call this method unless absolutely necessary...

    def get_headers(self, delete=0):

        if len(self.data) == 0:
            return []
        #;

        if self.hdrs:
            # the headers have been loaded
            return self.hdrs
        #;

        self.m.info("reading %d headers... " % len(self.data))
        hdrdict = shelve.open(self.hdrdictpath,'c')
        for f in self.data:

            rpmfile = os.path.join(self.path,f)
            r = rpmfile2.RPMfile(rpmfile)
            
            # we are using the shelved dictionary hdrdict to convert
            # from actual filenames to the yum header file path.

            if isinstance(f, unicode):
                f = f.encode("utf-8")
            #;


            if not hdrdict.has_key(f):
                # no entry, create one...
                try:
                    r.open()
                    self.hdrs.append((f,r))
                    hdrfile = os.path.join(self.hdrdir,r.get_id()+".hdr")
                    if isinstance(hdrfile, unicode):
                        hdrfile = hdrfile.encode("utf-8")
                    #;
                    hdrdict[f] = hdrfile
                    hdrdict[hdrfile] = f
                    r.write(hdrfile)
                    self.m.debug("header cached in %s\n"%hdrfile)
                except rpmfile2.RPMerror:
                    self.m.failure("problem with file %s\n" %f)
                    self.death_row.append((f,"rpm error"))
                #;
                continue
            #;

            hdrfile = hdrdict[f]

            if util.isnewer(rpmfile, hdrfile) > 1 or not os.path.exists(hdrfile):
                # we must update the hdrfile
                try:
                    r.open()
                    self.hdrs.append((f,r))
                    hdrfile = os.path.join(self.hdrdir,r.get_id()+".hdr")
                    if isinstance(hdrfile, unicode):
                        hdrfile = hdrfile.encode("utf-8")
                    #;
                    hdrdict[f] = hdrfile
                    hdrdict[hdrfile] = f
                    r.write(hdrfile)
                    self.m.debug("header re-cached in %s\n"%hdrfile)
                except:
                    self.m.failure("problem with file %s\n" %f)
                    self.death_row.append((f,"rpm error"))
                #;
                continue
            else:
                # load header from the cached file
                try:
                    r.read(hdrfile)
                    self.hdrs.append((f,r))
                    self.m.debug("header read from cache %s\n"%hdrfile)
                except:
                    self.m.failure("problem with file %s\n" %f)
                    self.death_row.append((f,"rpm error"))
                #;
            #;
        hdrdict.close()
        self.m.append("done.\n")

        # Delete files...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            # or put them on death row
            self.shelve("death_row", self.death_row)
        #;
        return self.hdrs
        #;
    #;

    # Destroy the release
    def destroy(self):

        if not self.path or not os.path.exists(self.path):
            return
        #;
        
        for f in self.data:
            ff = os.path.join(self.path,f)
            if os.path.exists(ff):
                os.unlink(ff)
            #;
        #;

        for f in self.files:
            if os.path.exists(f):
                os.unlink(f)
            #;
        #;
    #;

    # Return number of RPMs in the directory...

    def size(self):
        return len(self.data)
    #;

    # Clean out all regular non-rpm files not in 'files'

    def clean(self,delete):
        L = os.listdir(self.path)
        files = map(lambda x: os.path.join(self.path,x), L)
        for f in files:
            if f not in self.data + self.files:
                ff = os.path.join(self.path,f)
                reason = "not owned by me (RPMdir)"
                if os.path.isdir(f):
                    what = "directory"
                else:
                    what = "file"
                #;
                self.m.warning("convicted %s %s, %s\n" % (what,ff,reason))
                self.death_row.append((ff,reason))
            #;
        #;

        # Delete files...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            # or put them on death row
            self.shelve("death_row", self.death_row)
        #;
        return self.data+self.files
    #;

    # Purge the directory of rpms where a newer version of the same name
    # exists. For safety reasons, this method does not actually unlink any
    # files, but puts them on death row to be murdered later.

    def purge(self,delete):
        import sys
        import rpm
        
        self.m.info("Purge %s\n" % self.path)
        D = {}
        same_list = []

        reason = "newer version exists"
        for r in self.get_headers():
            # get_headers returns a list of tuples of (filename, header)
            h = r[1]
            ff = r[0]
            name =  h["NAME"]+h["ARCH"]

            if D.has_key(name):

                # We already encountered this package. Check which is more
                # recent.

                h2 = D[name][1]
                ff2 = D[name][0]
                cmp = rpm.versionCompare(h.data, h2.data)
                if cmp == 1:
                    D[name] = r  # ff is newer than ff2
                    self.m.warning("convicted file %s, %s\n" % (ff2,reason))
                    self.death_row.append((ff2,reason))
                elif cmp == 0:
                    same_list.append(ff2)
                elif cmp == -1:
                    # ff is older than ff2
                    self.m.warning("convicted file %s, %s\n" % (ff,reason))
                    self.death_row.append((ff,reason))
                #;
                #;
            else:
                D[name] = r
            #;
        #;

        # Next we purge files that are in the wrong place, like src rpms in
        # the binaries directory. This is catastrofic for the repo and can
        # happen for example if the mirror is incorrectly defined.
        
        for r in self.get_headers():
            arch = r[1]["ARCH"]
            issource = r[1]["SOURCEPACKAGE"]
            ff = r[0]
            
            if self.issource:
                if not issource:
                    # a binary rpm in an srpm directory
                    reason = "bin/src conflict"
                    self.m.warning("convicted file %s, %s\n" % (ff,reason))
                    self.death_row.append((ff,reason))
                #;

            else:
                if arch not in self.archs:
                    reason = "arch conflict"
                    self.death_row.append((ff,reason))
                    self.m.warning("convicted file %s, %s\n" % (ff,reason))
                #;
                if issource:
                    # a src rpm in a binary rpm directory
                    reason = "src/bin conflict"
                    self.m.warning("convicted file %s, %s\n" % (ff,reason))
                    self.death_row.append((ff,reason))
                #;
            #;
        #;
        self.m.info("%d files can be purged\n" % len(self.death_row))

        # Delete files on death row...
        if delete:
            util.delete(self.m,self.death_row)
        else:
            self.shelve("death_row", self.death_row)
        #;

        return
    #;

    # Generate web pages with a detailed report on each package
    # in the directory...more or less the same output as you get
    # with rpm -qi.

    def webgen(self, Document, htmldir):
        import HTMLgen, time
        import webgen

        if self.ignore: return
        
        for ff,h in self.get_headers():

            p = os.path.basename(ff)
            doc = Document(self,stylesheet=self.repo.stylesheet)
            doc.header(self.repo.id)
            doc.append(HTMLgen.Href("index_"+self.parent.id+doc.suffix,"[Up]"))
            doc.append(HTMLgen.Heading(2,p))

            # Make the table
            table = HTMLgen.TableLite(Class="table", width="60%")

            # Some rogue characters make htmlgen croak. Actually, the
            # rpm interface is in error, because the module returns an
            # ascii string, even if it contains characters >127.
            
            license = util.fixrogue(h["LICENSE"])
            summary = util.fixrogue(h["SUMMARY"])

            if h["PACKAGER"]:
                packager = util.fixrogue(h["PACKAGER"])
            else:
                packager = ''
            #;

            # Circumvent a bug in the rpm python bindings: packager
            # has type str, but may contain non-ascii characters

            packager = h["PACKAGER"]
            if packager:
                try:
                    packager = unicode(packager, "ascii")
                except UnicodeError:
                    try:
                        packager = unicode(packager, "utf-8")
                    except UnicodeError:
                        packager = unicode(packager, "iso-8859-1")
                    #;
                #;
            else:
                packager = ''
            #;
            packager = util.fixrogue(packager)

            L = [("Name",h["NAME"],"Distribution",h["DISTRIBUTION"]),
                 ("Version", h["VERSION"], "Architecture", h["ARCH"]),
                 ("Release", h["RELEASE"], "Build time", time.ctime(h["BUILDTIME"])),
                 ("Group", h["GROUP"], "Build host", h["BUILDHOST"]),
                 ("Size", h["SIZE"], "License", HTMLgen.RawText(license)),
                 ("Packager", HTMLgen.RawText(packager)),("URL", HTMLgen.Href(url=h["URL"],text=h["URL"])),
                 ("Summary", HTMLgen.RawText(summary))]

            for line in L:
                row = HTMLgen.TR()
                if len(line) > 2:
                    row.append(HTMLgen.TD(line[0],Class="col1"))
                    row.append(HTMLgen.TD(line[1],Class="col2"))
                    row.append(HTMLgen.TD(line[2],Class="col1"))
                    row.append(HTMLgen.TD(line[3],Class="col2"))
                else:
                    row.append(HTMLgen.TD(line[0],Class="col1"))
                    row.append(HTMLgen.TD(line[1],Class="col2",colspan=3))
                #;
                table.append(row)
            #;
            row = HTMLgen.TR()

            # Translate rogue characters again...
            description = util.fixrogue(h["DESCRIPTION"])

            row.append(HTMLgen.TD(HTMLgen.RawText(description), Class="col2", colspan=4))
            table.append(row)
            
            doc.append(HTMLgen.Paragraph(table))
            doc.footer()
            doc.write(os.path.join(htmldir,p)+doc.suffix)
        #;
        return
    #;


    # A careful check on the content of the self.path directory

    def audit (self):
        import md5

        # Make sure all files are in list of rpms
        for f in  glob.glob(os.path.join(self.path, "*.rpm")):
            if f not in self.data:
                self.m.debug ("entry %s added to file list\n" % f)
                self.data.append(f)
            #;
        self.data.sort()

        md5dict = shelve.open(self.md5dictpath,'c')
        
        # Now, check md5
        for f in self.data:
            if isinstance(f, unicode):
                f = f.encode("utf-8")
            #;
            if not md5dict.has_key(f):
                md5calc = md5.new()
                fd = open(f)
                md5calc.update(fd.read())
                siz = fd.tell()
                fd.close()
                hex =(md5calc.hexdigest())
                md5dict[f] = (hex,siz)
            #;
        #;
        md5dict.sync()

# rpm/md5 integrety check not implemented  yet... :-(

        md5dict.close()

    #;

#;

####
